import 'package:additional_exceptions/additional_exceptions.dart';
import 'package:decimal/decimal.dart';

extension DecimalExtension on Decimal {
  // This is modified method Decimal.toStringAsFixed()
  String toStringAsFixedFloor(int fractionDigits) {
    assert(fractionDigits >= 0);
    if (fractionDigits == 0) return round().toBigInt().toString();
    final value = floor(scale: fractionDigits);
    final intPart = value.toBigInt();
    final decimalPart = (Decimal.one + value.abs() - intPart.abs().toDecimal())
        .shift(fractionDigits);
    return '$intPart.${decimalPart.toString().substring(1)}';
  }

  static Decimal? fromDynamic(dynamic value) {
    if (value == null) return null;
    if (value is int) return Decimal.fromInt(value);
    if (value is String) return Decimal.parse(value);
    throw BadTypeException('Bad type ${value.runtimeType} of $value value');
  }

  static Decimal? fromDynamicE(dynamic dynamicValue,
      {String? name,
      Decimal? defaultValue,
      Set<Decimal>? allowableValues,
      bool nullAllowed = false}) {
    final value =
        dynamicValue == null ? defaultValue : fromDynamic(dynamicValue);
    if (value == null && !nullAllowed) {
      throw NotAllowableValueException(
          'The null value${_ofName(name)}is not allowed');
    }
    if (allowableValues == null || allowableValues.isEmpty) return value;
    if (allowableValues.contains(value)) return value;
    throw NotAllowableValueException('The $value value${_ofName(name)}is not '
        'one of the allowable ones: $allowableValues');
  }

  static Decimal? fromMapEntry(Map map, dynamic key,
          {Decimal? defaultValue,
          Set<Decimal>? allowableValues,
          bool nullAllowed = false}) =>
      fromDynamicE(map[key],
          name: key,
          defaultValue: defaultValue,
          allowableValues: allowableValues,
          nullAllowed: nullAllowed);

  static String _ofName(String? name) =>
      name == null || name == '' ? ' ' : ' of $name ';
}
